package com.ipoint.rentme.app.utils;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FilenameFilter;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.sf.jasperreports.engine.JREmptyDataSource;
import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JRPrintPage;
import net.sf.jasperreports.engine.JasperFillManager;
import net.sf.jasperreports.engine.JasperPrint;
import net.sf.jasperreports.engine.JasperReport;

/**
 * The compiler obtains all report files with common prefix. Loads, compiles and fills them all into single {@link JasperReport}. 
 * 
 * @author aectann
 *
 */
public class MultipageReportCompiler implements ReportCompiler {

	private static final String JASPER_EXTENSION = ".jasper";
	private static final String REPORTS_PATH = "/reports";
	private Map<String, List<File>> cachedReports = new HashMap<String, List<File>>();

	@SuppressWarnings("unchecked")
	@Override
	public JasperPrint getCompiledAndFilledReport(final String prefix, Map<String, Object> parameters) {
		JasperPrint result = null;
		try {
			List<File> reportsToFill;
			if (cachedReports.containsKey(prefix)) {
				reportsToFill = cachedReports.get(prefix);
			} else {
				reportsToFill = new ArrayList<File>();
				File reportsDir = new File(getClass().getResource(REPORTS_PATH).toURI());
				FilenameFilter filter = new FilenameFilter() {
					@Override
					public boolean accept(File dir, String name) {
						return name.startsWith(prefix)&&name.endsWith(JASPER_EXTENSION);
					}
				};
				File[] compiledReportFiles = reportsDir.listFiles(filter);
				Arrays.sort(compiledReportFiles, new Comparator<File>(){
					@Override
					public int compare(File o1, File o2) {
						return o1.getName().compareTo(o2.getName());
					}					
				});
				for (File file : compiledReportFiles) {
					reportsToFill.add(file);
				}
				cachedReports.put(prefix, reportsToFill);
			}
			List<JasperPrint> prints = new ArrayList<JasperPrint>();
			for (File jasperReport : reportsToFill) {
				prints.add(JasperFillManager.fillReport(new FileInputStream(jasperReport), parameters, new JREmptyDataSource()));
			}
			JasperPrint finalPrint = prints.get(0);
			for (JasperPrint jasperPrint : prints) {
				if(jasperPrint.equals(finalPrint)){
					continue;
				}
				List pages = jasperPrint.getPages();
				for (Object page : pages) {
					finalPrint.addPage((JRPrintPage) page);
				}
			}
			result = finalPrint;
		} catch (URISyntaxException e) {
			// never happens
			e.printStackTrace();
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (JRException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return result;
	}

}
