package com.ipoint.rentme.components;

import org.apache.commons.lang.StringUtils;
import org.apache.tapestry5.RenderSupport;
import org.apache.tapestry5.SymbolConstants;
import org.apache.tapestry5.annotations.AfterRender;
import org.apache.tapestry5.annotations.Environmental;
import org.apache.tapestry5.ioc.annotations.Inject;
import org.apache.tapestry5.ioc.annotations.Symbol;

/**
 * The component embeds google analytics scripts into the page. Only works in production mode.
 * {@link #GOOGLE_ANALYTICS_ID} system symbol must be specified - see contributeApplicationDefaults method of your
 * *.services.YourAppNameModule class.
 * 
 * @author aectann
 */
public class GoogleAnalytics {

	public static final String GOOGLE_ANALYTICS_ID = "gaid";

	@Environmental
	private RenderSupport renderSupport;

	@Inject
	@Symbol(SymbolConstants.PRODUCTION_MODE)
	private boolean production;

	@Inject
	@Symbol(GOOGLE_ANALYTICS_ID)
	private String googleAnalyticsId;

	void setupRender() {
		if (embedNeeded()) {
			renderSupport.addClasspathScriptLink("com/ipoint/rentme/components/GoogleAnalytics.js");
		}
	}

	private boolean embedNeeded() {
		return production && StringUtils.isNotBlank(googleAnalyticsId);
	}

	@AfterRender
	void afterRender() {
		if (embedNeeded()) {
			renderSupport.addScript(String.format(
					"try {var pageTracker = _gat._getTracker('%s');pageTracker._trackPageview();} catch(err) {}",
					googleAnalyticsId));
		}
	}
}
