package com.ipoint.rentme.dao;

import java.util.List;

import com.ipoint.rentme.model.BaseObject;

public interface BaseDao<T extends BaseObject> {
	/**
	 * Retrieve object by it's identifier.
	 * 
	 * @param id - object's identifier.
	 * @return retrieved object or null if there is no object of such type with such identifier.
	 */
	T get(Long id);

	/**
	 * Stores object state into persistence storage. If object is null new identifier will set.
	 * 
	 * @param objectToSave - object to save.
	 * @return saved object.
	 */
	T save(T objectToSave);

	/**
	 * Delete object from database.
	 * 
	 * @param id - identifier of object to remove.
	 */
	void delete(Long id);

	/**
	 * retrieves all objects of such type from persistence storage. TODO Do not use in production, just for debugging.
	 * 
	 * @return list of retrieved objects.
	 */
	@Deprecated
	List<T> getAll();

	/**
	 * @return total count of all active objects.
	 */
	Long getCount();
}
