package com.ipoint.rentme.dao.hibernate;

import java.lang.reflect.ParameterizedType;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import org.apache.log4j.Logger;
import org.hibernate.Query;
import org.hibernate.SessionFactory;
import org.springframework.orm.hibernate3.HibernateTemplate;
import org.springframework.orm.hibernate3.support.HibernateDaoSupport;

import com.ipoint.rentme.dao.BaseDao;
import com.ipoint.rentme.model.BaseObject;

/**
 * Base dao for all our daos.
 * 
 * @author Tchernykh Alexey (tchernykh@gmail.com)
 */
public abstract class AbstractHibernateDao<T extends BaseObject> extends HibernateDaoSupport implements BaseDao<T> {
	
	private final Class<T> persistentClass;

	protected Class<T> getPersistentClass() {
		return this.persistentClass;
	}

	@Override
	protected HibernateTemplate createHibernateTemplate(SessionFactory sessionFactory) {
		HibernateTemplate result = super.createHibernateTemplate(sessionFactory);
		result.setAllowCreate(false);
		return result;
	}

	@SuppressWarnings("unchecked")
	public AbstractHibernateDao() {
		this.persistentClass = (Class<T>) ((ParameterizedType) this.getClass().getGenericSuperclass())
				.getActualTypeArguments()[0];
	}

	public void delete(Long id) {
		this.getSession().delete(this.get(id));
	}

	@SuppressWarnings("unchecked")
	public T get(Long id) {
		Query query = this.getSession().createQuery("from " + this.persistentClass.getName() + " o where o.id = :id");
		query.setLong("id", id);
		return (T) query.uniqueResult();
	}

	protected Logger getLogger() {
		return Logger.getLogger(this.getClass().getSimpleName());
	}

	
	@SuppressWarnings("unchecked")
    public List<T> getLoadAll() {
        return super.getHibernateTemplate().loadAll(this.persistentClass);
    }
	
	
	/**
	 * That is very bad code style, do not use the method.
	 * 
	 * @see com.ipoint.persistence.AbstractDao#getAll()
	 */
	@SuppressWarnings("unchecked")
	@Deprecated
	public List<T> getAll() {
		// TODO - think that this way can lead to poor performance
		// TODO - Konstanin: eliminate usage of the method in future.
		// throw new UnsupportedOperationException(
		// "That is very bad code style, do not use the method.");
		try {
			Query q = this.getSession().createQuery(
					"select distinct o from " + this.persistentClass.getName() + " o order by o.id desc");
			return q.list();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return Collections.EMPTY_LIST;
	}

	public T save(T objectToSave) {
		if (objectToSave.isNew()) {
			objectToSave.setCreatedDate(new Date());
		}
		objectToSave.setLastUpdatedDate(new Date());
		// actually this should set id to new object
		getSession().saveOrUpdate(objectToSave);
		return objectToSave;
	}

	public Long getCount() {
		return (Long) getNamedQuery(this.persistentClass.getSimpleName() + ".count").uniqueResult();
	}

	@SuppressWarnings("unchecked")
	protected T getFirstResult(String queryName, Object... parameters) {
		Query q = getNamedQuery(queryName, parameters);
		q.setMaxResults(1);
		return (T) q.uniqueResult();
	}

	protected Query getNamedQuery(String queryName, Object... parameters) {
		Query q = getSession().getNamedQuery(queryName);
		if (parameters.length % 2 != 0) {
			throw new IllegalStateException("count of parameters should be even");
		}
		for (int i = 0; i < parameters.length; i += 2) {
			if (parameters[i] instanceof String) {
				q.setParameter((String) parameters[i], parameters[i + 1]);
			} else {
				throw new IllegalStateException("name of parameter should be a String");
			}
		}
		return q;
	}
}