package com.ipoint.rentme.dao.hibernate;

import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.hibernate.Query;
import org.springframework.beans.factory.annotation.Autowired;

import com.ipoint.rentme.dao.AdvertisementDao;
import com.ipoint.rentme.dao.AuditRecordDao;
import com.ipoint.rentme.model.Action;
import com.ipoint.rentme.model.Advertisement;
import com.ipoint.rentme.model.AdvertisementType;
import com.ipoint.rentme.model.AuditRecord;
import com.ipoint.rentme.model.Districts;
import com.ipoint.rentme.model.PaymentMethods;
import com.ipoint.rentme.model.SearchRooms;
import com.ipoint.rentme.model.User;

public class AdvertisementDaoHibernate extends AbstractHibernateDao<Advertisement> implements AdvertisementDao {

	@Autowired
	private AuditRecordDao auditRecordDao;

	public AuditRecordDao getAuditRecordDao() {
		return auditRecordDao;
	}

	public void setAuditRecordDao(AuditRecordDao auditRecordDao) {
		this.auditRecordDao = auditRecordDao;
	}

	@Override
	@SuppressWarnings("unchecked")
	public List<Advertisement> findAdvertisement(Districts district, SearchRooms roomCount, AdvertisementType type,
			boolean excludeFirstFloor, Integer priceMax, boolean balcony, boolean exludeLastFloor, boolean telephone,
			PaymentMethods paymentMethod) {
		StringBuilder q = new StringBuilder("from Advertisement where approved=true");
		if (district != null) {
			q.append(" and district='").append(district).append("'");
		}
		if (roomCount != null) {
			q.append(" and roomsCount='").append(roomCount).append("'");
		}
		if (type != null) {
			q.append(" and type='").append(type).append("'");
		}
		if (paymentMethod != null) {
			q.append(" and paymentMethod='").append(paymentMethod).append("'");
		}
		if (excludeFirstFloor) {
			q.append(" and floor<>1");
		}
		if (priceMax != null) {
			q.append(" and price<=").append(priceMax * 1000);
		}
		if (exludeLastFloor) {
			q.append(" and floor<>maxFloor");
		}
		if (balcony) {
			q.append(" and balcony=true");
		}
		if (telephone) {
			q.append(" and telephone=true");
		}
		q.append(" and activeTill >= :currentDate");
		q.append(" order by createdDate desc");
		Query query = getSession().createQuery(q.toString());
		query.setParameter("currentDate", new Date());
		return query.list();
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<Advertisement> getUnmoderatedList() {
		return getNamedQuery("Advertisement.unmoderated").list();
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<Advertisement> getUnactualList() {
		return getNamedQuery("Advertisement.unactual").list();
	}

	@Override
	public int getLastNumber() {
		Query q = getSession().createQuery("select max(number) from Advertisement");
		return q.uniqueResult() != null ? (Integer) q.uniqueResult() : 0;
	}

	@Override
	public Advertisement getNextToModerate() {
		List<Advertisement> unmoderatedList = getUnmoderatedList();
		if (unmoderatedList.size() > 0) {
			return unmoderatedList.get(0);
		}
		List<Advertisement> unactualList = getUnactualList();
		if (unactualList.size() > 0) {
			return unactualList.get(0);
		}
		return null;
	}

	@Override
	public Advertisement save(Advertisement advertisement) {
		Advertisement save;
		if (advertisement.isNew()) {
			synchronized (this) {
				advertisement.setNumber(getLastNumber() + 1);
				Calendar c = Calendar.getInstance();
				c.add(Calendar.DAY_OF_YEAR, Advertisement.DEFAULT_ACTIVE_DAYS);
				advertisement.setActiveTill(c.getTime());
				save = super.save(advertisement);
			}
		} else {
			save = super.save(advertisement);
		}
		return save;
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<Advertisement> getActualList(User user) {
		return getNamedQuery("Advertisement.actualByOwner", "owner", user).list();
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<Advertisement> getUnactualList(User user) {
		return getNamedQuery("Advertisement.unactualByOwner", "owner", user).list();
	}

	@Override
	public Advertisement saveWithAuditInfo(Advertisement objectToSave, String remoteAddress, User user) {
		Advertisement saved = save(objectToSave);
		AuditRecord auditRecord = new AuditRecord();
		auditRecord.setUser(user);
		auditRecord.setAction(saved.isApproved() == null ? Action.CREATE : saved.isApproved() ? Action.APPROVE
				: Action.REJECT);
		auditRecord.setRemoteAddress(remoteAddress);
		auditRecord.setAdvertisement(saved);
		auditRecordDao.save(auditRecord);
		return saved;
	}

}