package com.ipoint.rentme.model;

import java.util.Calendar;
import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;

import org.apache.tapestry5.beaneditor.DataType;
import org.apache.tapestry5.beaneditor.NonVisual;
import org.apache.tapestry5.beaneditor.Validate;
import org.apache.tapestry5.beaneditor.Width;

@Entity
@NamedQueries( {
		@NamedQuery(name = "Advertisement.unmoderated", query = "from Advertisement where approved is null"),
		@NamedQuery(name = "Advertisement.unactual", query = "from Advertisement where approved = true and activeTill<current_timestamp()"),
		@NamedQuery(name = "Advertisement.actualByOwner", query = "from Advertisement where activeTill>current_timestamp() and owner = :owner and approved = true"),
		@NamedQuery(name = "Advertisement.unactualByOwner", query = "from Advertisement where activeTill<current_timestamp() and owner = :owner and approved = true") })
public class Advertisement extends BaseObject {

	private static final long serialVersionUID = 1627153560326220783L;
	
	private int floor;
	private int maxFloor;
	private int price;
	private Rooms roomsCount;
	private Boolean approved;
	private String description;
	private Districts district;
	private Date activeTill;
	private String phoneNumber;
	private String contactName;
	private boolean telephone;
	private boolean balcony;
	private PaymentMethods paymentMethod;
	private int number;
	private User owner;
	private AdvertisementType type;

	@Column(length = 4)
	@Enumerated(EnumType.STRING)
	@Validate("required")
	public AdvertisementType getType() {
		return type;
	}

	public void setType(AdvertisementType type) {
		this.type = type;
	}

	public static final int DEFAULT_ACTIVE_DAYS = 5;

	@ManyToOne
	@NonVisual
	public User getOwner() {
		return owner;
	}

	public void setOwner(User owner) {
		this.owner = owner;
	}

	@NonVisual
	public int getNumber() {
		return number;
	}

	public void setNumber(int number) {
		this.number = number;
	}

	@Enumerated(EnumType.STRING)
	@Column(length = 6)
	public PaymentMethods getPaymentMethod() {
		return paymentMethod;
	}

	public void setPaymentMethod(PaymentMethods paymentMethod) {
		this.paymentMethod = paymentMethod;
	}

	public boolean isTelephone() {
		return telephone;
	}

	public void setTelephone(boolean telephone) {
		this.telephone = telephone;
	}

	public boolean isBalcony() {
		return balcony;
	}

	public void setBalcony(boolean balcony) {
		this.balcony = balcony;
	}

	@Validate("required")
	@Width(7)
	public int getPrice() {
		return price;
	}

	public void setPrice(int price) {
		this.price = price;
	}

	@Validate("required,maxlength=24")
	@Column(length = 24)
	public String getPhoneNumber() {
		return phoneNumber;
	}

	public void setPhoneNumber(String phoneNumber) {
		this.phoneNumber = phoneNumber;
	}

	@Column(length = 64)
	@Width(32)
	@Validate("required,maxlength=64")
	public String getContactName() {
		return contactName;
	}

	public void setContactName(String contactName) {
		this.contactName = contactName;
	}
	
	@Validate("required,min=1,max=20")
	@Width(7)
	public int getFloor() {
		return floor;
	}

	public void setFloor(int floor) {
		this.floor = floor;
	}
	
	@Validate("required,min=1,max=20")
	@Width(7)
	public int getMaxFloor() {
		return maxFloor;
	}

	public void setMaxFloor(int maxFloor) {
		this.maxFloor = maxFloor;
	}

	@NonVisual
	public Boolean isApproved() {
		return approved;
	}

	public void setApproved(Boolean approved) {
		this.approved = approved;
	}

	@Validate("required,maxlength=255")
	@DataType("longtext")
	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	@Enumerated(EnumType.STRING)
	@Validate("required")
	@Column(length = 3)
	public Districts getDistrict() {
		return district;
	}

	public void setDistrict(Districts district) {
		this.district = district;
	}

	@NonVisual
	public Date getActiveTill() {
		return activeTill;
	}

	public void setActiveTill(Date activeTill) {
		this.activeTill = activeTill;
	}

	@Validate("required")
	@Enumerated(EnumType.STRING)
	@Column(length = 5)
	public Rooms getRoomsCount() {
		return roomsCount;
	}

	public void setRoomsCount(Rooms roomsCount) {
		this.roomsCount = roomsCount;
	}
	
	public void markActive() {
		Calendar c = Calendar.getInstance();
		c.add(Calendar.DAY_OF_YEAR, Advertisement.DEFAULT_ACTIVE_DAYS);
		setActiveTill(c.getTime());
	}
}