package com.ipoint.rentme.model;

import java.util.HashSet;
import java.util.Set;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.Transient;

import org.apache.tapestry5.beaneditor.DataType;
import org.apache.tapestry5.beaneditor.NonVisual;
import org.apache.tapestry5.beaneditor.Validate;
import org.hibernate.annotations.Cascade;
import org.hibernate.annotations.CollectionOfElements;
import org.hibernate.annotations.LazyCollection;
import org.hibernate.annotations.LazyCollectionOption;
import org.springframework.security.GrantedAuthority;
import org.springframework.security.userdetails.UserDetails;

@Entity
@NamedQueries( { @NamedQuery(name = "User.byUsername", query = "from User where username = :username"),
		@NamedQuery(name = "User.byEmail", query = "from User where email = :email") })
public class User extends BaseObject implements UserDetails {

	private static final long serialVersionUID = -8626856297560936040L;
	
	private String name;

	private String email;

	private String phone;

	private String description;
	
	private String password;

	private String username;

	private boolean enabled;

	private Set<Role> roles;
	
	private User() {
		// for hibernate only
	}

	public User(Role firstrole, Role... roles) {
		this();
		if (firstrole == null && roles.length < 1) {
			throw new IllegalArgumentException("You must specify at least one role for new user");
		}
		Set<Role> userRoles = new HashSet<Role>();
		userRoles.add(firstrole);
		for (Role role : roles) {
			userRoles.add(role);
		}
		setRoles(userRoles);
	}

	@Override
	@Transient
	public GrantedAuthority[] getAuthorities() {
		GrantedAuthority[] result = new GrantedAuthority[getRoles().size()];
		int i = 0;
		for (final Role role : getRoles()) {
			result[i] = new GrantedAuthority() {
				private static final long serialVersionUID = 7951877964536816127L;

				@Override
				public String getAuthority() {
					return role.toString();
				}

				@Override
				public int compareTo(Object o) {
					GrantedAuthority a = (GrantedAuthority) o;
					return Role.valueOf(getAuthority()).compareTo(Role.valueOf(a.getAuthority()));
				}
			};
			i++;
		}
		return result;
	}

	@Column(length = 512)
	@Validate("maxlength=512")
	@DataType("longtext")
	public String getDescription() {
		return description;
	}

	@Column(length = 32, unique = true)
	@Validate("required,maxlength=32")
	public String getEmail() {
		return email;
	}

	@Column(length = 64)
	@Validate("required,maxlength=64")
	public String getName() {
		return name;
	}

	
	@Override
	@Column(length = 40)
	@Validate("required,maxlength=40")
	public String getPassword() {
		return password;
	}

	@Column(length = 16)
	@Validate("required,maxlength=16")
	public String getPhone() {
		return phone;
	}

	@CollectionOfElements
	@JoinTable(name = "UserRoles", joinColumns = { @JoinColumn(name = "userId") })
	@Cascade(value = org.hibernate.annotations.CascadeType.DELETE_ORPHAN)
	@Enumerated(value = EnumType.STRING)
	@Column(name = "role", length = 14)
	@LazyCollection(LazyCollectionOption.FALSE)
	@NonVisual
	public Set<Role> getRoles() {
		return roles;
	}
	
	@Override
	@Column(length = 24, unique = true)
	@Validate("required,maxlength=24")
	public String getUsername() {
		return username;
	}
	
	@Override
	@Transient
	public boolean isAccountNonExpired() {
		return true;
	}
	
	@Override
	@Transient
	public boolean isAccountNonLocked() {
		return true;
	}

	@Override
	@Transient
	public boolean isCredentialsNonExpired() {
		return true;
	}

	@Override
	@NonVisual
	public boolean isEnabled() {
		return enabled;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	public void setEnabled(boolean enabled) {
		this.enabled = enabled;
	}

	public void setName(String name) {
		this.name = name;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public void setPhone(String phone) {
		this.phone = phone;
	}

	public void setRoles(Set<Role> roles) {
		this.roles = roles;
	}

	public void setUsername(String username) {
		this.username = username;
	}

}
