package com.ipoint.rentme.pages;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.util.HashMap;

import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JRExporterParameter;
import net.sf.jasperreports.engine.JasperExportManager;
import net.sf.jasperreports.engine.JasperPrint;
import net.sf.jasperreports.engine.export.JRHtmlExporter;
import net.sf.jasperreports.engine.export.JRRtfExporter;

import org.apache.tapestry5.StreamResponse;
import org.apache.tapestry5.annotations.Persist;
import org.apache.tapestry5.ioc.annotations.Inject;

import com.ipoint.rentme.app.utils.HtmlStreamResponse;
import com.ipoint.rentme.app.utils.NullPxStreamResponse;
import com.ipoint.rentme.app.utils.PdfStreamResponse;
import com.ipoint.rentme.app.utils.ReportCompiler;
import com.ipoint.rentme.app.utils.RtfStreamResponse;

/**
 * Returns a formed contract. Either rtf or pdf or html format. 
 * 
 * @author aectann
 *
 */
public class Contract {
	
	private static final String CONTRACT_KEY = "contract";
	
	@Persist("flash")
	private com.ipoint.rentme.model.Contract contract;
	
	public com.ipoint.rentme.model.Contract getContract() {
		return contract;
	}

	public void setContract(com.ipoint.rentme.model.Contract contract) {
		this.contract = contract;
	}

	@Inject
	private ReportCompiler reportCompiler;
	
	StreamResponse onActivate() throws JRException{
		return getResponse("pdf");
	}
	
	StreamResponse onActivate(String type) throws JRException{
		return getResponse(type);
		
	}

	private StreamResponse getResponse(String type) throws JRException {
		HashMap<String, Object> parameters = new HashMap<String, Object>();
		if(contract != null){
			parameters.put(CONTRACT_KEY, contract);
		}else{
			parameters.put(CONTRACT_KEY, new com.ipoint.rentme.model.Contract());
		}
		
		JasperPrint reportPrint = reportCompiler.getCompiledAndFilledReport(CONTRACT_KEY, parameters);
		if("pdf".equals(type)){
			return new PdfStreamResponse(new ByteArrayInputStream(JasperExportManager.exportReportToPdf(reportPrint)));
		}else if("rtf".equals(type)){
			JRRtfExporter rtfExporter = new JRRtfExporter();
			rtfExporter.setParameter(JRExporterParameter.JASPER_PRINT, reportPrint);
			ByteArrayOutputStream rtfOutput = new ByteArrayOutputStream();
			rtfExporter.setParameter(JRExporterParameter.OUTPUT_STREAM, rtfOutput);
			rtfExporter.exportReport();
			return new RtfStreamResponse(new ByteArrayInputStream(rtfOutput.toByteArray()));
		}else if("html".equals(type)){
			JRHtmlExporter htmlExporter = new JRHtmlExporter();
			htmlExporter.setParameter(JRExporterParameter.JASPER_PRINT, reportPrint);
			ByteArrayOutputStream rtfOutput = new ByteArrayOutputStream();
			htmlExporter.setParameter(JRExporterParameter.OUTPUT_STREAM, rtfOutput);
			htmlExporter.exportReport();
			return new HtmlStreamResponse(new ByteArrayInputStream(rtfOutput.toByteArray()));
		}else{
			return new NullPxStreamResponse();
		}
	}
}
