package com.ipoint.rentme.pages;

import java.net.MalformedURLException;

import org.apache.tapestry5.annotations.Component;
import org.apache.tapestry5.annotations.Persist;
import org.apache.tapestry5.corelib.components.Form;
import org.apache.tapestry5.ioc.Messages;
import org.apache.tapestry5.ioc.annotations.Inject;

import com.ipoint.rentme.dao.DuplicateEmailException;
import com.ipoint.rentme.dao.DuplicateUsernameException;
import com.ipoint.rentme.model.Role;
import com.ipoint.rentme.model.User;
import com.ipoint.rentme.services.AuthService;
import com.ipoint.rentme.services.ExternalLinkFactory;
import com.ipoint.rentme.services.MailService;

public abstract class UserEdit {
	
	public AuthService getAuthService() {
		return authService;
	}

	public ExternalLinkFactory getLinkFactory() {
		return linkFactory;
	}

	@Inject
	private AuthService authService;
	
	@Inject
	private MailService mailService;
	
	public MailService getMailService() {
		return mailService;
	}

	@Inject
	private ExternalLinkFactory linkFactory;
	
	
	@Component
	private Form form;
	
	@Persist
	private User user;
	
	private String passwordCheck;
	
	protected void onActivate() {
		if (user == null) {
			user = new User(Role.ROLE_AGENCY);
			getForm().clearErrors();
		}
	}

	public User getUser() {
		return user;
	}

	public void setUser(User user) {
		this.user = user;
	}
	
	public String getPasswordCheck() {
		return passwordCheck;
	}

	public void setPasswordCheck(String passwordCheck) {
		this.passwordCheck = passwordCheck;
	}
		
	@Inject
	private Messages messages;
	
	protected Object onSuccess() throws MalformedURLException {
		if (!getUser().getPassword().equals(getPasswordCheck())) {
			getForm().recordError(getMessages().get("error.passwords-do-not-match"));
			return this;
		}
		try {
			saveUser();
		} catch (DuplicateEmailException e) {
			getForm().recordError(getMessages().get("error.duplicate-email"));
			return this;
		} catch (DuplicateUsernameException e) {
			getForm().recordError(getMessages().get("error.duplicate-username"));
			return this;
		}
		sendEmail();
		user = null;
		return getSuccessPage();
	}

	protected abstract void saveUser() throws DuplicateEmailException, DuplicateUsernameException;
	
	protected Messages getMessages() {
		return messages;
	}

	protected Form getForm() {
		return form;
	}

	protected abstract Object getSuccessPage();

	protected void sendEmail() {
		// Nothing sent by default.
	}
	
	public String getTitle() {
		return messages.get(getClass().getSimpleName().toLowerCase());
	}
}
