/**
 * 
 */
package com.ipoint.rentme.pages;

import java.text.Format;

import org.apache.tapestry5.PersistenceConstants;
import org.apache.tapestry5.annotations.InjectPage;
import org.apache.tapestry5.annotations.Persist;
import org.apache.tapestry5.ioc.Messages;
import org.apache.tapestry5.ioc.annotations.Inject;

import com.ipoint.rentme.app.utils.CodeValidateException;
import com.ipoint.rentme.app.utils.CodeValidator;
import com.ipoint.rentme.app.utils.Formats;
import com.ipoint.rentme.dao.AdvertisementDao;
import com.ipoint.rentme.model.Advertisement;

/**
 * @author happy
 */
public class View {

	@Inject
	private AdvertisementDao advertisementDao;

	@InjectPage
	private Missing missing;

	public Object onActivate(long advId) {
		advertisement = advertisementDao.get(advId);
		if (advertisement != null && advertisement.isApproved()) {
			return null;
		}
		return missing;
	}

	@Persist
	private Advertisement advertisement;

	@Inject
	private Messages messages;

	private String code;

	@Persist(PersistenceConstants.FLASH)
	private boolean success;

	@Persist(PersistenceConstants.FLASH)
	private String errorMessage;

	public String getErrorMessage() {
		return errorMessage;
	}

	public void setErrorMessage(String errorMessage) {
		this.errorMessage = errorMessage;
	}

	public boolean isSuccess() {
		return success;
	}

	public void setSuccess(boolean success) {
		this.success = success;
	}

	public String getCode() {
		return code;
	}

	public void setCode(String code) {
		this.code = code;
	}

	public Advertisement getAdvertisement() {
		return advertisement;
	}

	public void setAdvertisement(Advertisement advertisement) {
		this.advertisement = advertisement;
	}

	public Object onSubmitFromBillingForm() {
		try {
			setSuccess(CodeValidator.validate(getCode(), this.messages));
		} catch (CodeValidateException e) {
			setSuccess(false);
			setErrorMessage(e.getMessage());
		}
		return this;
	}

	public Format getFormat() {
		return Formats.DATE_FORMAT;
	}

	public String getExtras() {
		String result = "Есть";
		boolean punkt = false;
		if (advertisement.isTelephone()) {
			result += " телефон";
			punkt = true;
		}
		if (advertisement.isBalcony()) {
			if (punkt) {
				result += ",";
			}
			result += " балкон";
			punkt = true;
		}
		return result + ".";
	}

	public boolean isHasExtras() {
		return getAdvertisement().isBalcony() || getAdvertisement().isTelephone();
	}

	public String getDistrictName() {
		return messages.get(advertisement.getDistrict().toString());
	}

	public String getRoomsCount() {
		return messages.get(advertisement.getRoomsCount().toString());
	}

	public String getPaymentMethod() {
		return advertisement.getPaymentMethod() != null ? messages.get(advertisement.getPaymentMethod().toString())
				: null;
	}

	public boolean isShowFull() {
		return isSuccess() || isHasOwner();
	}

	public boolean isHasOwner() {
		return getAdvertisement().getOwner() != null;
	}
}