package com.ipoint.rentme.pages.moderation;

import javax.servlet.http.HttpServletRequest;

import org.apache.tapestry5.annotations.Component;
import org.apache.tapestry5.annotations.InjectPage;
import org.apache.tapestry5.annotations.Persist;
import org.apache.tapestry5.corelib.components.Form;
import org.apache.tapestry5.ioc.Messages;
import org.apache.tapestry5.ioc.annotations.Inject;

import com.ipoint.rentme.dao.AdvertisementDao;
import com.ipoint.rentme.model.Advertisement;
import com.ipoint.rentme.services.AuthService;

/**
 * Page for moderator.
 */
public class Moderation {
	
	public void onActivate(long advId) {
		if (advId > 0) {
			advertisement = advertisementDao.get(advId);
		}
	}
	
    @Persist
	private Advertisement advertisement;
        
	@Inject
	private AdvertisementDao advertisementDao;

	public Advertisement getAdvertisement() {
		return advertisement;
	}
	
	public void setAdvertisement(Advertisement advertisement) {
		this.advertisement = advertisement;
	}
	
	@InjectPage
	private Unmoderated unmoderated;
	
	@InjectPage
	private Moderation moderation;
	
	@Component
	private Form form;

	@Inject
	private Messages messages;
	
	@Inject
	private HttpServletRequest request;
	
	@Inject
	private AuthService authService;
	
	public Object onSuccess() {
		// HACK we should use custom validator instead..
		if (advertisement.getFloor() > advertisement.getMaxFloor()) {
			form.recordError(messages.get("error.maxfloor"));
			return this;
		} 
		advertisementDao.saveWithAuditInfo(advertisement, request.getRemoteHost(), authService.getCurrentUser());
		advertisement = advertisementDao.getNextToModerate();
		if (advertisement == null) {
			return unmoderated;
		}
		moderation.setAdvertisement(advertisement);
		return moderation;
		
	}

	public void onSelectedFromApprove() {
		advertisement.setApproved(true);
		advertisement.markActive();
	}
	
	public Object onActionFromDiscard() {
		advertisement.setApproved(false);
		return onSuccess();
	}

}