package com.ipoint.rentme.services.impl;

import javax.annotation.PostConstruct;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.security.context.SecurityContextHolder;
import org.springframework.security.providers.encoding.PasswordEncoder;
import org.springframework.security.userdetails.UserDetails;
import org.springframework.security.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;

import com.ipoint.rentme.dao.DuplicateEmailException;
import com.ipoint.rentme.dao.DuplicateUsernameException;
import com.ipoint.rentme.dao.UserDao;
import com.ipoint.rentme.model.Role;
import com.ipoint.rentme.model.User;
import com.ipoint.rentme.services.AuthService;

@Service("authService")
public class AuthServiceImpl implements AuthService {
	
	private static final String ADMIN_DEFAULT_USERNAME = "admin";
	private static final String ADMIN_DEAFULT_PASSWORD = "a1d3m3i7n";
	
	@Autowired
	private UserDao userDao;
	
	@Autowired
	private PasswordEncoder passwordEncoder;
	
	@PostConstruct
	void init() throws DuplicateEmailException, DuplicateUsernameException {
		User admin = userDao.getUser(ADMIN_DEFAULT_USERNAME);
		if (admin == null) {
			User adminUser = new User(Role.ROLE_ADMIN, Role.ROLE_MODERATOR);
			// TODO externalize em all..
			adminUser.setUsername(ADMIN_DEFAULT_USERNAME);
			adminUser.setPassword(ADMIN_DEAFULT_PASSWORD);
			adminUser.setEmail("info@ipoint.ru");
			adminUser.setPhone("996-176");
			adminUser.setName("Администратор");
			adminUser.setEnabled(true);
			createUser(adminUser);
		}
	}
	
	@Override
	public User getCurrentUser() {
		Object user = SecurityContextHolder.getContext().getAuthentication().getPrincipal();
		User result = null;
		if (user instanceof User) {
			result = userDao.get(((User) user).getId());
		}
		return result;
	}

	@Override
	public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException, DataAccessException {
		return userDao.getUser(username);
	}
	
	@Override
	public User createUser(User user) throws DuplicateEmailException, DuplicateUsernameException {
		if (!user.isNew()) {
			throw new IllegalArgumentException(
					"This method can be used only for creating users, use updateCurrentUser method if you want to save user.");
		}
		User existingUser = userDao.getUser(user.getUsername());
		if (existingUser != null) {
			throw new DuplicateUsernameException(user.getUsername());
		}
		existingUser = (User) userDao.getUserByEmail(user.getEmail());
		if (existingUser != null) {
			throw new DuplicateEmailException(user.getEmail());
		}
		user.setPassword(passwordEncoder.encodePassword(user.getPassword(), user.getUsername()));
		return userDao.save(user);
	}

	@Override
	public User updateCurrentUser(String username, String password, String email, String phone, String name)
			throws DuplicateEmailException, DuplicateUsernameException {
		User currentUser = getCurrentUser();
		if (currentUser == null) {
			throw new IllegalStateException("No logged in user.");
		}
		User existingUser = userDao.getUser(username);
		if (!existingUser.equals(currentUser)) {
			throw new DuplicateUsernameException(username);
		}
		existingUser = userDao.getUserByEmail(email);
		if (!existingUser.equals(currentUser)) {
			throw new DuplicateEmailException(username);
		}
		currentUser.setUsername(username);
		currentUser.setEmail(email);
		currentUser.setPhone(phone);
		currentUser.setEmail(email);
		currentUser.setPassword(passwordEncoder.encodePassword(password, username));
		currentUser.setName(name);
		return userDao.save(currentUser);
	}

}
