/**
 * 
 */
package com.ipoint.rentme.services.impl;

import java.util.HashMap;
import java.util.Map;

import javax.mail.MessagingException;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

import org.apache.log4j.Logger;
import org.apache.velocity.app.VelocityEngine;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.mail.javamail.MimeMessageHelper;
import org.springframework.ui.velocity.VelocityEngineUtils;

import com.ipoint.rentme.TemplateConstant;
import com.ipoint.rentme.model.User;
import com.ipoint.rentme.services.MailService;

/**
 * @author happy
 *
 */
public class MailServiceImpl implements MailService {

	private static final String REGISTRATION_CONFIRMATION = "registration-confirmation";

	private static final String REGISTRATION_CONFIRMATION_TITLE = "registration-confirmation-title";

	private static final String TEMPLATE_POSTFIX = ".vm";

	private static final String ENCODING = "UTF-8";

	private static final String EMAIL_TEMPLATES_FOLDER = "/templates/email/";

	private static final String INVITE = "invite";

	private static final String INVITE_TITLE = "invite-title";

	@Autowired
	private JavaMailSender mailSender;
	
	@Autowired
	private VelocityEngine velocityEngine;

	/**
	 * Sender address used in outgoing emails.
	 */
	private String defaultSender;
	
	private String defaultReceivers;
	
	private static final Logger logger = Logger.getLogger(MailServiceImpl.class);
	
	public void setMailSender(JavaMailSender mailSender) {
		this.mailSender = mailSender;
	}

	public JavaMailSender getMailSender() {
		return this.mailSender;
	}

	private String getDefaultSender() {
		return this.defaultSender;
	}

	public void setDefaultSender(String defaultSender) {
		this.defaultSender = defaultSender;
	}
	
	public String getDefaultReceivers() {
		return defaultReceivers;
	}

	public void setDefaultReceivers(String defaultReceivers) {
		this.defaultReceivers = defaultReceivers;
	}

	public void sendMessage(String recipient, String subject, String body) {
		try {
			sendMessage(recipient, subject, body, null);
		} catch (MessagingException e) {
			logger.error("Error while trying to send an email to " + recipient + ", subject - " + subject, e);
		}
	}
	
	private void sendMessage(String recipient, String subject, String body, String replyToAddress)
			throws MessagingException {
		MimeMessage mimeMessage = this.getMailSender().createMimeMessage();
		MimeMessageHelper helper = new MimeMessageHelper(mimeMessage, ENCODING);
		if (replyToAddress != null) {
			mimeMessage.setReplyTo(new InternetAddress[] { new InternetAddress(replyToAddress) });
		}
		helper.setFrom(new InternetAddress(this.getDefaultSender()));
		helper.setSubject(subject);
		helper.setTo(new InternetAddress(recipient));
		helper.setText(body, true);
		try {
			this.getMailSender().send(mimeMessage);
		} catch (Exception e) {
			//TODO do something to send it later..
		}
		
	}

	@Override
	public void notifyAdministrator(String message) {
		String[] drs = getDefaultRecipients();
		for (String recipient : drs) {
			sendMessage(recipient, "RentMe", message);
		}
	}

	private String[] getDefaultRecipients() {
		String dr = getDefaultReceivers();
		String[] drs = dr.split(",");
		return drs;
	}

	@Override
	public void registrationMail(User user, String activationLink) {
		Map<String, Object> model = new HashMap<String, Object>();
		model.put(TemplateConstant.USER.toString(), user);
		model.put(TemplateConstant.LINK.toString(), activationLink);
		sendMessage(user.getEmail(), mergeTemplate(model, REGISTRATION_CONFIRMATION_TITLE), mergeTemplate(model, REGISTRATION_CONFIRMATION));
	}

	private String mergeTemplate(Map<String, Object> model, String template) {
		String title = VelocityEngineUtils.mergeTemplateIntoString(velocityEngine,
				EMAIL_TEMPLATES_FOLDER
				+ template
				+ TEMPLATE_POSTFIX, ENCODING, model);
		return title;
	}

	@Override
	public void inviteMail(User user, String link, String unencodedPassword) {
		Map<String, Object> model = new HashMap<String, Object>();
		model.put(TemplateConstant.USER.toString(), user);
		model.put(TemplateConstant.LINK.toString(), link);
		model.put(TemplateConstant.PASSWORD.toString(), unencodedPassword);
		sendMessage(user.getEmail(), mergeTemplate(model, INVITE_TITLE), mergeTemplate(model, INVITE));
	}

	@Override
	public void sendFeedBack(String subject, String message, String address) {
		for (String recipient : getDefaultRecipients()) {
			try {
				sendMessage(recipient, subject, message, address);
			} catch (MessagingException e) {
				logger.error("Error while tryin to send feedback message from " + address + ", subject is " + subject
						+ ", message is\n" + message, e);
			}
		}
	}
}